@@ -163,7 +163,7 @@ gdk_pixbuf__svg_image_stop_load (gpointer data, GError **error)
                 result = FALSE;
         }
 
-        rsvg_handle_free (context->handle);
+        g_object_unref (G_OBJECT(context->handle));
         g_free (context);
 
         return TRUE;
@@ -191,12 +191,25 @@ fill_vtable (GdkPixbufModule *module)
 #define GDK_PIXBUF_FORMAT_THREADSAFE (1 << 2)
 #endif
 
+#ifndef GDK_PIXBUF_CHECK_VERSION
+#define GDK_PIXBUF_CHECK_VERSION(major,minor,micro)    \
+    (GDK_PIXBUF_MAJOR > (major) || \
+     (GDK_PIXBUF_MAJOR == (major) && GDK_PIXBUF_MINOR > (minor)) || \
+     (GDK_PIXBUF_MAJOR == (major) && GDK_PIXBUF_MINOR == (minor) && \
+      GDK_PIXBUF_MICRO >= (micro)))
+#endif
+
 void
 fill_info (GdkPixbufFormat *info)
 {
-        static GdkPixbufModulePattern signature[] = {
-                { "<svg", NULL, 100 },
-                { "<!DOCTYPE svg", NULL, 100 },
+        static GdkPixbufModulePattern signature_old[] = {
+                { (unsigned char*) "<svg", NULL, 100 },
+                { (unsigned char*) "<!DOCTYPE svg", NULL, 100 },
+                { NULL, NULL, 0 }
+        };
+        static GdkPixbufModulePattern signature_new[] = {
+                { (unsigned char*) " <svg", (unsigned char*) "*    ", 100 },
+                { (unsigned char*) " <!DOCTYPE svg", (unsigned char*) "*             ", 100 },
                 { NULL, NULL, 0 }
         };
         static gchar *mime_types[] = { /* yes folks, i actually have run into all of these in the wild... */
@@ -215,7 +228,11 @@ fill_info (GdkPixbufFormat *info)
         };
 
         info->name        = "svg";
-        info->signature   = signature;
+        if (GDK_PIXBUF_CHECK_VERSION (2, 7, 4)) {
+                info->signature = signature_new;
+        } else {
+                info->signature = signature_old;
+        }
         info->description = _("Scalable Vector Graphics");
         info->mime_types  = mime_types;
         info->extensions  = extensions;
