@@ -25,6 +25,8 @@
 
 #include "config.h"
 #include "rsvg-css.h"
+#include "rsvg-private.h"
+#include "rsvg-styles.h"
 
 #include <glib.h>
 #include <stdio.h>
@@ -39,6 +41,9 @@
 #define MM_PER_INCH     (25.4)
 #define PICA_PER_INCH   (6.0)
 
+#define SETINHERIT() G_STMT_START {if (inherit != NULL) *inherit = TRUE;} G_STMT_END
+#define UNSETINHERIT() G_STMT_START {if (inherit != NULL) *inherit = FALSE;} G_STMT_END
+
 /**
  * rsvg_css_parse_vbox
  * @vbox: The CSS viewBox
@@ -47,48 +52,42 @@
  * @w: The Width output
  * @h: The Height output
  *
- * Returns: Success or failure
+ * Returns: 
  */
-gboolean
-rsvg_css_parse_vbox (const char * vbox, double * x, double * y,
-					 double * w, double * h)
+RsvgViewBox
+rsvg_css_parse_vbox (const char * vbox)
 {
+	RsvgViewBox vb;
 	gdouble * list;
 	guint list_len;
+	vb.active = FALSE;
+
+	vb.x = vb.y = 0; vb.w = vb.h = 0;
 
 	list = rsvg_css_parse_number_list(vbox, &list_len);
 
 	if(!(list && list_len))
-		return FALSE;
+		return vb;
 	else if(list_len != 4) {
 		g_free(list);
-		return FALSE;
+		return vb;
 	} 
 	else {
-		*x = list[0];
-		*y = list[1];
-		*w = list[2];
-		*h = list[3];
+		vb.x = list[0];
+		vb.y = list[1];
+		vb.w = list[2];
+		vb.h = list[3];
+		vb.active = TRUE;
 
 		g_free(list);
-		return TRUE;
+		return vb;
 	}
 }
 
 
-/**
- * rsvg_css_parse_length: Parse CSS2 length to a pixel value.
- * @str: Original string.
- * @pixels_per_inch: Pixels per inch
- * @fixed: Where to store boolean value of whether length is fixed.
- *
- * Parses a CSS2 length into a pixel value.
- *
- * Returns: returns the length.
- **/
-double
-rsvg_css_parse_length (const char *str, gdouble pixels_per_inch, 
-					   gint *percent, gint *em, gint *ex)
+static double
+rsvg_css_parse_raw_length (const char *str, gint *in, 
+						   gint *percent, gint *em, gint *ex)
 {
 	double length = 0.0;
 	char *p = NULL;
@@ -111,15 +110,27 @@ rsvg_css_parse_length (const char *str, gdouble pixels_per_inch,
 	if (p && (strcmp(p, "px") != 0))
 		{
 			if (!strcmp(p, "pt"))
-				length *= (pixels_per_inch / POINTS_PER_INCH);
+				{
+					length /= POINTS_PER_INCH;
+					*in = TRUE;
+				}
 			else if (!strcmp(p, "in"))
-				length *= pixels_per_inch;
+				*in = TRUE;
 			else if (!strcmp(p, "cm"))
-				length *= (pixels_per_inch / CM_PER_INCH);
+				{
+					length /= CM_PER_INCH;
+					*in = TRUE;
+				}
 			else if (!strcmp(p, "mm"))
-				length *= (pixels_per_inch / MM_PER_INCH);
+				{
+					length /= MM_PER_INCH;
+					*in = TRUE;
+				}
 			else if (!strcmp(p, "pc"))
-				length *= (pixels_per_inch / PICA_PER_INCH);
+				{
+					length /= PICA_PER_INCH;
+					*in = TRUE;				
+				}
 			else if (!strcmp(p, "em"))
 				*em = TRUE;
 			else if (!strcmp(p, "ex"))
@@ -134,33 +145,79 @@ rsvg_css_parse_length (const char *str, gdouble pixels_per_inch,
 	return length;
 }
 
-/**
- * rsvg_css_parse_normalized_length: Parse CSS2 length to a pixel value.
- * @str: Original string.
- * @pixels_per_inch: Pixels per inch
- * @normalize_to: Bounding box's width or height, as appropriate, or 0
- *
- * Parses a CSS2 length into a pixel value.
- *
- * Returns: returns the length.
- */
-double
-rsvg_css_parse_normalized_length(const char *str, gdouble pixels_per_inch,
-								 gdouble width_or_height, gdouble font_size)
+RsvgLength
+_rsvg_css_parse_length(const char *str)
 {
-	double length;
-	gint percent, em, ex;
-	percent = em = ex = FALSE;
+	RsvgLength out;
+	gint percent, em, ex, in;
+	percent = em = ex = in = FALSE;
 	
-	length = rsvg_css_parse_length (str, pixels_per_inch, &percent, &em, &ex);
+	out.length = rsvg_css_parse_raw_length (str, &in, &percent, &em, &ex);
 	if (percent)
-		return length * width_or_height;
+		out.factor = 'p';
 	else if (em)
-		return length * font_size;
+		out.factor = 'm';
 	else if (ex)
-		return length * font_size * 2.0; /* stolen from imagemagick */
+		out.factor = 'x';
+	else if (in)
+		out.factor = 'i';
 	else
-		return length;
+		out.factor = '\0';
+	return out;
+}
+
+double
+_rsvg_css_normalize_length(const RsvgLength * in, RsvgDrawingCtx * ctx, 
+								  char dir)
+{
+	if (in->factor == '\0')
+		return in->length;
+	else if (in->factor == 'p')
+		{
+			if (dir == 'h')
+				return in->length * ctx->vb.w;
+			if (dir == 'v')
+				return in->length * ctx->vb.h;
+			if (dir == 'o')
+				return in->length * rsvg_viewport_percentage(ctx->vb.w, ctx->vb.h);
+		}
+	else if (in->factor == 'm' || in->factor == 'x')
+		{
+			double font = _rsvg_css_hand_normalize_length(&rsvg_state_current(ctx)->font_size,
+																 ctx->dpi_y, ctx->vb.h, 1);	
+			if (in->factor == 'm')
+				return in->length * font;
+			else
+				return in->length * font / 2.;
+		}
+	else if (in->factor == 'i')
+		{
+			if (dir == 'h')
+				return in->length * ctx->dpi_x;
+			if (dir == 'v')
+				return in->length * ctx->dpi_y;
+			if (dir == 'o')
+				return in->length * rsvg_viewport_percentage(ctx->dpi_x, ctx->dpi_y);
+		}
+	return 0;
+}
+
+double
+_rsvg_css_hand_normalize_length(const RsvgLength * in, gdouble pixels_per_inch,
+									   gdouble width_or_height, gdouble font_size)
+{
+	if (in->factor == '\0')
+		return in->length;
+	else if (in->factor == 'p')
+		return in->length * width_or_height;
+	else if (in->factor == 'm')
+		return in->length * font_size;
+	else if (in->factor == 'x')
+		return in->length * font_size / 2.;
+	else if (in->factor == 'i')
+		return in->length * pixels_per_inch;
+
+	return 0;
 }
 
 gboolean
@@ -239,10 +296,12 @@ rsvg_css_color_compare (const void * a, const void * b)
  * Parse a CSS2 color specifier, return RGB value
  */
 guint32
-rsvg_css_parse_color (const char *str, guint32 inherit)
+rsvg_css_parse_color (const char *str, gboolean * inherit)
 {
 	gint val = 0;
-	
+
+	SETINHERIT ();
+
 	if (str[0] == '#')
 		{
 			int i;
@@ -276,7 +335,7 @@ rsvg_css_parse_color (const char *str, guint32 inherit)
 			
 			if (strstr (str, "%") != 0)
 				{
-					gint i, nb_toks;
+					guint i, nb_toks;
 					char ** toks;
 
 					/* assume rgb (9%, 100%, 23%) */
@@ -315,7 +374,7 @@ rsvg_css_parse_color (const char *str, guint32 inherit)
 			val = PACK_RGB (r,g,b);
 		}
 	else if (!strcmp (str, "inherit"))
-		val = inherit;
+		UNSETINHERIT ();
 	else 
 		{
 			const static ColorPair color_list [] =
@@ -476,7 +535,10 @@ rsvg_css_parse_color (const char *str, guint32 inherit)
 			
 			/* default to black on failed lookup */
 			if (result == NULL)
-				val = 0;
+				{
+					UNSETINHERIT ();
+					val = 0;
+				}
 			else
 				val = result->rgb;
 		}
@@ -592,8 +654,10 @@ rsvg_css_parse_time (const char * str)
 }
 
 PangoStyle
-rsvg_css_parse_font_style (const char * str, PangoStyle inherit)
+rsvg_css_parse_font_style (const char * str, gboolean * inherit)
 {
+	SETINHERIT ();
+
 	if (str)
 		{
 			if (!strcmp(str, "oblique"))
@@ -601,27 +665,38 @@ rsvg_css_parse_font_style (const char * str, PangoStyle inherit)
 			if (!strcmp(str, "italic"))
 				return PANGO_STYLE_ITALIC;
 			else if (!strcmp(str, "inherit"))
-				return inherit;
+				{
+					UNSETINHERIT ();
+					return PANGO_STYLE_NORMAL;
+				}
 		}
+	UNSETINHERIT ();
 	return PANGO_STYLE_NORMAL;
 }
 
 PangoVariant
-rsvg_css_parse_font_variant (const char * str, PangoVariant inherit)
+rsvg_css_parse_font_variant (const char * str, gboolean * inherit)
 {
+	SETINHERIT ();
+
 	if (str)
     {
 		if (!strcmp(str, "small-caps"))
 			return PANGO_VARIANT_SMALL_CAPS;
 		else if (!strcmp(str, "inherit"))
-			return inherit;
+			{
+				UNSETINHERIT ();
+				return PANGO_VARIANT_NORMAL;
+			}
     }
+	UNSETINHERIT ();
 	return PANGO_VARIANT_NORMAL;
 }
 
 PangoWeight
-rsvg_css_parse_font_weight (const char * str, PangoWeight inherit)
+rsvg_css_parse_font_weight (const char * str, gboolean * inherit)
 {
+	SETINHERIT ();
 	if (str)
 		{
 			if (!strcmp (str, "lighter"))
@@ -649,15 +724,21 @@ rsvg_css_parse_font_weight (const char * str, PangoWeight inherit)
 			else if (!strcmp (str, "900"))
 				return (PangoWeight)900;
 			else if (!strcmp(str, "inherit"))
-				return inherit;
+				{
+					UNSETINHERIT ();
+					return PANGO_WEIGHT_NORMAL;
+				}
 		}
-	
+
+	UNSETINHERIT ();	
 	return PANGO_WEIGHT_NORMAL; 
 }
 
 PangoStretch
-rsvg_css_parse_font_stretch (const char * str, PangoStretch inherit)
+rsvg_css_parse_font_stretch (const char * str, gboolean * inherit)
 {
+	SETINHERIT ();
+
 	if (str)
 		{
 			if (!strcmp (str, "ultra-condensed"))
@@ -677,23 +758,32 @@ rsvg_css_parse_font_stretch (const char * str, PangoStretch inherit)
 			else if (!strcmp (str, "ultra-expanded"))
 				return PANGO_STRETCH_ULTRA_EXPANDED;
 			else if (!strcmp(str, "inherit"))
-				return inherit;
-		}
+				{
+					UNSETINHERIT ();
+					return PANGO_STRETCH_NORMAL;
+				}
+		}	
+	UNSETINHERIT ();
 	return PANGO_STRETCH_NORMAL;
 }
 
 const char *
-rsvg_css_parse_font_family (const char * str, const char * inherit)
+rsvg_css_parse_font_family (const char * str, gboolean * inherit)
 {
+	SETINHERIT ();
+
 	if (!str)
 		return NULL;	
 	else if (!strcmp (str, "inherit"))
-		return inherit;
+		{
+			UNSETINHERIT ();
+			return NULL;
+		}
 	else
 		return str;
 }
 
-#if !defined(HAVE_STRTOK_R) /* && !GLIB_CHECK_VERSION(2, 3, 2) */
+#if !defined(HAVE_STRTOK_R)
 
 static char *
 strtok_r(char *s, const char *delim, char **last)
@@ -734,24 +824,6 @@ strtok_r(char *s, const char *delim, char **last)
 gchar **
 rsvg_css_parse_list(const char * in_str, guint * out_list_len)
 {
-	/* the following code is defective because it creates blank entries when two splitting chars are next to each other*/
-#if 0 /* GLIB_CHECK_VERSION(2, 3, 2) */
-
-	gchar ** string_array;
-	guint n;
-
-	/* this may fix bug #113538 */
-
-	string_array = g_strsplit_set(in_str, ", \t\n", -1);
-
-	for(n = 0; string_array[n] != NULL; n++)
-		;
-
-	*out_list_len = n;
-	return string_array;
-
-#else
-
 	char *ptr, *tok;
 	char *str;
 
@@ -794,8 +866,6 @@ rsvg_css_parse_list(const char * in_str, guint * out_list_len)
 	}
 
 	return string_array;
-
-#endif
 }
 
 gdouble *
@@ -846,7 +916,8 @@ rsvg_css_parse_number_optional_number(const char * str,
     *y = *x;
 }
 
-int rsvg_css_parse_aspect_ratio(const char * str)
+int 
+rsvg_css_parse_aspect_ratio (const char * str)
 {
 	char ** elems;
 	guint nb_elems;
@@ -869,7 +940,7 @@ int rsvg_css_parse_aspect_ratio(const char * str)
 				else if (!strcmp (elems[i], "xMinYMid"))
 					ratio = RSVG_ASPECT_RATIO_XMIN_YMID;
 				else if (!strcmp (elems[i], "xMidYMid"))
-					ratio = RSVG_ASPECT_RATIO_XMIN_YMID;
+					ratio = RSVG_ASPECT_RATIO_XMID_YMID;
 				else if (!strcmp (elems[i], "xMaxYMid"))
 					ratio = RSVG_ASPECT_RATIO_XMAX_YMID;
 				else if (!strcmp (elems[i], "xMinYMax"))
@@ -887,3 +958,16 @@ int rsvg_css_parse_aspect_ratio(const char * str)
 
 	return ratio;
 }
+
+gboolean 
+rsvg_css_parse_overflow(const char * str, gboolean * inherit)
+{
+	SETINHERIT ();
+	if (!strcmp(str, "visible") || !strcmp(str, "auto"))
+		return 1;
+	if (!strcmp(str, "hidden") || !strcmp(str, "scroll"))
+		return 0;
+	UNSETINHERIT ();
+	return 0;
+}
+
